import os

from typing import Generator

from .entry import RequirementsEntry, RequirementsEntrySource
from .parse_requirements_list import parse_requirements_list


def parse_requirements_txt(
    file_path: str,
) -> Generator[RequirementsEntry, None, None]:
    source = RequirementsEntrySource(
        path=os.path.realpath(file_path), line=None, line_number=None
    )

    with open(file_path, "r") as fp:
        for requirement in parse_requirements_list(source, fp.readlines()):
            yield requirement



from typing import Generator, List

from .entry import (
    RequirementsEditableEntry,
    RequirementsEntry,
    RequirementsRecursiveEntry,
)
from .identify_package_list_file_type import (
    PackageListFileType,
    identify_package_list_file_type,
)
from .parse_requirements_txt import parse_requirements_txt
from .parse_setup_py import parse_setup_py


def list_packages_from_files(
    file_paths: List[str],
    recurse_recursive: bool = False,
    recurse_editable: bool = False,
) -> Generator[RequirementsEntry, None, None]:
    generators = []

    for file_path in file_paths:
        package_list_file_type = identify_package_list_file_type(file_path)
        if package_list_file_type == PackageListFileType.REQUIREMENTS_TXT:
            generators.append(parse_requirements_txt(file_path))
        elif package_list_file_type == PackageListFileType.SETUP_PY:
            generators.append(parse_setup_py(file_path))

    while len(generators) > 0:
        for requirement in generators[0]:
            if isinstance(requirement, RequirementsRecursiveEntry):
                if recurse_recursive:
                    generators.append(
                        parse_requirements_txt(requirement.absolute_path)
                    )
                else:
                    yield requirement
            elif isinstance(requirement, RequirementsEditableEntry):
                if recurse_editable:
                    generators.append(
                        parse_setup_py(
                            requirement.resolved_absolute_path,
                            requirement.extras,
                        )
                    )
                else:
                    yield requirement
            else:
                yield requirement

        generators = generators[1:]



import os
import re

from typing import Generator, List, Optional, Tuple

from .entry import (
    RequirementsDirectRefEntry,
    RequirementsEditableEntry,
    RequirementsEntry,
    RequirementsEntrySource,
    RequirementsPackageEntry,
    RequirementsRecursiveEntry,
    RequirementsVCSPackageEntry,
    RequirementsWheelPackageEntry,
)


class RequirementsEntryParseError(RuntimeError):
    pass


def parse_requirements_list(
    source: Optional[RequirementsEntrySource], lines: List[str]
) -> Generator[RequirementsEntry, None, None]:

    for index, line in enumerate(lines):
        stripped_line = _clean_line(line)

        if not len(stripped_line) or stripped_line.startswith("#"):
            continue

        line_source = None
        if source:
            line_source = RequirementsEntrySource(
                path=source.path, line=stripped_line, line_number=index + 1
            )

        requirement, extras, markers = _split_requirement_and_markers(
            stripped_line
        )

        if requirement.startswith("-r"):
            yield parse_recursive_requirements_entry(
                line_source, requirement, extras, markers
            )

        elif requirement.startswith("-e"):
            yield parse_editable_requirements_entry(
                line_source, requirement, extras, markers
            )

        elif re.match(r"^(.+)\+", requirement):
            yield parse_vcs_requirements_entry(
                line_source, requirement, extras, markers
            )

        elif "@" in requirement:
            yield parse_direct_ref_requirements_entry(
                line_source, requirement, extras, markers,
            )

        elif requirement.endswith("whl"):
            yield parse_wheel_requirements_entry(
                line_source, requirement, extras, markers
            )

        else:
            yield parse_package_requirements_entry(
                line_source, requirement, extras, markers
            )


def parse_recursive_requirements_entry(
    source: RequirementsEntrySource,
    requirement: str,
    extras: List[str],
    markers: Optional[str] = None,
) -> RequirementsRecursiveEntry:
    original_path = _clean_line(requirement.replace("-r", ""))
    absolute_path = os.path.realpath(
        os.path.join(os.path.dirname(source.path), original_path)
    )

    return RequirementsRecursiveEntry(
        source=source, original_path=original_path, absolute_path=absolute_path,
    )


def parse_editable_requirements_entry(
    source: RequirementsEntrySource,
    requirement: str,
    extras: List[str],
    markers: Optional[str] = None,
) -> RequirementsEditableEntry:
    original_path = _clean_line(requirement.replace("-e", ""))
    resolved_path = original_path

    if not original_path.endswith(".py"):
        resolved_path = os.path.join(original_path, "setup.py")

    absolute_path = os.path.realpath(
        os.path.join(os.path.dirname(source.path), original_path)
    )
    resolved_absolute_path = os.path.realpath(
        os.path.join(os.path.dirname(source.path), resolved_path)
    )

    return RequirementsEditableEntry(
        source=source,
        original_path=original_path,
        absolute_path=absolute_path,
        resolved_path=resolved_path,
        resolved_absolute_path=resolved_absolute_path,
        extras=extras,
    )


def parse_vcs_requirements_entry(
    source: RequirementsEntrySource,
    requirement: str,
    extras: List[str],
    markers: Optional[str] = None,
) -> RequirementsVCSPackageEntry:
    vcs_uri_split = requirement.split("+")
    vcs = vcs_uri_split[0]

    uri_tag_split = vcs_uri_split[1].split("#")
    uri = uri_tag_split[0]

    tag = None
    if len(uri_tag_split) > 1:
        tag = uri_tag_split[1]

    return RequirementsVCSPackageEntry(source=source, vcs=vcs, uri=uri, tag=tag)


def parse_wheel_requirements_entry(
    source: RequirementsEntrySource,
    requirement,
    extras: List[str],
    markers: Optional[str] = None,
) -> RequirementsWheelPackageEntry:
    return RequirementsWheelPackageEntry(source=source, uri=requirement)


def parse_direct_ref_requirements_entry(
    source: RequirementsEntrySource,
    requirement,
    extras: List[str],
    markers: Optional[str] = None,
) -> RequirementsDirectRefEntry:
    name, uri = requirement.split("@")
    name = name.strip()
    uri = uri.strip()

    return RequirementsDirectRefEntry(
        source=source, name=name, uri=uri, markers=markers,
    )


def parse_package_requirements_entry(
    source: RequirementsEntrySource,
    requirement: str,
    extras: List[str],
    markers: Optional[str] = None,
) -> RequirementsPackageEntry:
    operators = ["==", ">=", ">", "<=", "<"]
    for operator in operators:
        parts = requirement.split(operator)
        if len(parts) == 1:
            continue

        return RequirementsPackageEntry(
            source=source,
            name=parts[0],
            extras=extras,
            operator=operator,
            version=parts[1],
            markers=markers,
        )

    return RequirementsPackageEntry(
        source=source, name=requirement, extras=extras, markers=markers,
    )


def _clean_line(line: str) -> str:
    return (
        line.strip()
        .replace("\n", "")
        .replace("\r", "")
        .replace("  ", " ")
        .replace("  ", " ")
    )


def _split_requirement_and_markers(
    line: str,
) -> Tuple[str, List[str], Optional[str]]:
    requirement = line
    markers = None
    extras = []

    if ";" in requirement:
        requirement, markers = requirement.split(";")

        requirement = requirement.strip()
        markers = markers.strip()

    extras_match = re.search(re.compile(r"\[.*\]"), requirement)
    if extras_match:
        matched_text = extras_match[0]

        requirement = requirement.replace(matched_text, "")
        extras = [
            extra.strip()
            for extra in matched_text.strip("[").strip("]").strip().split(",")
        ]

    return requirement, extras, markers



from .entry import (
    RequirementsEditableEntry,
    RequirementsEntry,
    RequirementsEntrySource,
    RequirementsPackageEntry,
    RequirementsRecursiveEntry,
    RequirementsVCSPackageEntry,
)
from .list_packages_from_files import list_packages_from_files
from .parse_requirements_list import (
    RequirementsEntryParseError,
    parse_requirements_list,
)
from .parse_requirements_txt import parse_requirements_txt
from .parse_setup_py import parse_setup_py

__all__ = [
    "parse_setup_py",
    "parse_requirements_txt",
    "parse_requirements_list",
    "list_packages_from_files",
    "RequirementsEntryParseError",
    "RequirementsEditableEntry",
    "RequirementsEntry",
    "RequirementsRecursiveEntry",
    "RequirementsVCSPackageEntry",
    "RequirementsEntrySource",
    "RequirementsPackageEntry",
    "parse_requirements",
]



import enum


class PackageListFileType(enum.Enum):
    REQUIREMENTS_TXT = "requirements.txt"
    SETUP_PY = "setup.py"


def identify_package_list_file_type(file_path: str) -> PackageListFileType:
    if file_path.endswith("setup.py"):
        return PackageListFileType.SETUP_PY

    return PackageListFileType.REQUIREMENTS_TXT



import argparse
import sys

from .entry import (
    RequirementsEditableEntry,
    RequirementsRecursiveEntry,
    RequirementsVCSPackageEntry,
    RequirementsWheelPackageEntry,
)
from .list_packages_from_files import list_packages_from_files


def main() -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--recurse-recursive",
        default=False,
        help="recurse into -r entries",
        action="store_true",
    )
    parser.add_argument(
        "--recurse-editable",
        default=False,
        help="recurse into -e entries",
        action="store_true",
    )
    parser.add_argument(
        "--dedupe",
        default=False,
        help="de-duplicate the resulting list",
        action="store_true",
    )
    parser.add_argument(
        "--remove-editable",
        default=False,
        help="remove editable requirements from the final list",
        action="store_true",
    )
    parser.add_argument(
        "--remove-recursive",
        default=False,
        help="remove recursive requirements from the final list",
        action="store_true",
    )
    parser.add_argument(
        "--remove-vcs",
        default=False,
        help="remove vcs requirements from the final list",
        action="store_true",
    )
    parser.add_argument(
        "--remove-wheel",
        default=False,
        help="remove wheel requirements from the final list",
        action="store_true",
    )
    parser.add_argument(
        "file_paths",
        nargs="+",
        help="list of requirements.txt or setup.py files",
    )

    args = parser.parse_args()

    reqs = list_packages_from_files(
        args.file_paths,
        recurse_recursive=args.recurse_recursive,
        recurse_editable=args.recurse_editable,
    )

    if args.remove_editable:
        reqs = [
            requirement
            for requirement in reqs
            if not isinstance(requirement, RequirementsEditableEntry)
        ]

    if args.remove_recursive:
        reqs = [
            requirement
            for requirement in reqs
            if not isinstance(requirement, RequirementsRecursiveEntry)
        ]

    if args.remove_vcs:
        reqs = [
            requirement
            for requirement in reqs
            if not isinstance(requirement, RequirementsVCSPackageEntry)
        ]

    if args.remove_wheel:
        reqs = [
            requirement
            for requirement in reqs
            if not isinstance(requirement, RequirementsWheelPackageEntry)
        ]

    if args.dedupe:
        deduped_reqs = list(set([str(requirement) for requirement in reqs]))
        for requirement in deduped_reqs:
            print(requirement)
    else:
        for requirement in reqs:
            print(requirement)

    return 0


if __name__ == "__main__":
    sys.exit(main())



import os

from dataclasses import dataclass, field
from typing import List, Optional


@dataclass
class RequirementsEntrySource:
    path: str
    line: Optional[str]
    line_number: Optional[str]


@dataclass
class RequirementsEntry:
    source: Optional[RequirementsEntrySource]


@dataclass
class RequirementsRecursiveEntry(RequirementsEntry):
    original_path: str
    absolute_path: str

    def __str__(self) -> str:
        path = os.path.relpath(self.absolute_path, os.getcwd())
        return f"-r {path}"


@dataclass
class RequirementsEditableEntry(RequirementsEntry):
    original_path: str
    absolute_path: str

    resolved_path: str
    resolved_absolute_path: str

    extras: List[str] = field(default_factory=list)

    def __str__(self) -> str:
        line = os.path.relpath(self.absolute_path, os.getcwd())
        if self.extras:
            line += "[" + ",".join(self.extras) + "]"

        return f"-e {line}"


@dataclass
class RequirementsVCSPackageEntry(RequirementsEntry):
    vcs: str
    uri: str
    tag: Optional[str]

    def __str__(self) -> str:
        result = f"{self.vcs}+{self.uri}"
        if self.tag:
            result += f"#{self.tag}"

        return result


@dataclass
class RequirementsWheelPackageEntry(RequirementsEntry):
    uri: str
    markers: Optional[str] = None

    def __str__(self) -> str:
        line = self.uri
        if self.markers:
            line += f" ; {self.markers}"

        return line


@dataclass
class RequirementsDirectRefEntry(RequirementsEntry):
    name: str
    uri: str
    markers: Optional[str] = None

    def __str__(self) -> str:
        line = f"{self.name} @ {self.uri}"
        if self.markers:
            line += f" ; {self.markers}"

        return line


@dataclass
class RequirementsPackageEntry(RequirementsEntry):
    name: str
    extras: List[str] = field(default_factory=list)
    operator: Optional[str] = None
    version: Optional[str] = None
    markers: Optional[str] = None

    def __str__(self) -> str:
        line = self.name

        if self.extras:
            line += "[" + ",".join(self.extras) + "]"

        if self.operator:
            line += self.operator

        if self.version:
            line += self.version

        if self.markers:
            line += f"; {self.markers}"

        return line



import os

from typing import Generator, List

import setuptools

from .entry import RequirementsEntry, RequirementsEntrySource
from .parse_requirements_list import parse_requirements_list


def parse_setup_py(
    file_path: str, extras: List[str] = []
) -> Generator[RequirementsEntry, None, None]:
    setup_kwargs = {}

    def _setup_proxy(*args, **kwargs):
        setup_kwargs.update(dict(kwargs))

    setuptools.setup = _setup_proxy

    with open(file_path, "r") as fp:
        exec(fp.read())

    source = RequirementsEntrySource(
        path=os.path.realpath(file_path), line=None, line_number=None
    )

    requirements = setup_kwargs.get("install_requires") or []

    extras_require = setup_kwargs.get("extras_require") or {}
    for extra_name, extra_requirements in extras_require.items():
        if extra_name not in extras:
            continue

        requirements.extend(extra_requirements)

    for requirement in parse_requirements_list(source, requirements):
        yield requirement
